library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity edge_detect is
    Port ( edclk : in std_logic;
           clk : in std_logic;
           edclk_fe : out std_logic;
           edclk_re : out std_logic;
           reset : in std_logic);
end edge_detect;

architecture Behavioral of edge_detect is
CONSTANT RESET_ACTIVE:std_logic:='1';
signal edclk_delayed,edclk_sync:std_logic;

begin
-- purpose	:syncs the external signal to the internal clk
--	type		:memorizing
--	inputs 	:reset,clo
--	outputs	: 
sync: process(reset,clk)
begin
	if (reset=RESET_ACTIVE) then
 		edclk_sync<='0';
	elsif rising_edge(clk) then
		edclk_sync<=edclk;
	end if;
end process;
-- purpose	:delays the synced signal for a edge detection
--	type		:memorizing
--	inputs 	:reset,clo
--	outputs	: 
clk_delay: process(reset,clk)
begin
	if reset=RESET_ACTIVE then
		edclk_delayed<='0';
	elsif rising_edge(clk) then
		edclk_delayed<=edclk_sync;
	end if;
end process;
-- purpose	:detects a falling edge of the signal
--	type		:memoryless
--	inputs 	:edclk_delayed,edclk_sync
--	outputs	:edclk_fe 
edclk_fe<='1' when (edclk_delayed='1' and edclk_sync='0') else '0';
-- purpose	:detects a rising edge of the signal
--	type		:memoryless
--	inputs 	:edclk_delayed,edclk_sync
--	outputs	:edclk_re 
edclk_re<='1' when (edclk_delayed='0' and edclk_sync='1') else '0';


end Behavioral;