library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity slave_input_sr is
    Port ( Data_out 	: out std_logic_vector(7 downto 0);
    		  clk   		: in std_logic;
		 	  sck_re 	: in std_logic;
		     reset  	: in std_logic;
			  sr_reset  : in std_logic;
           shift_enable : in std_logic;
           sin       :in  std_logic);
end slave_input_sr;

architecture Behavioral of slave_input_sr is

signal Data_int,Data_int_nxt:std_logic_vector(7 downto 0);					  
signal shift_in:std_logic;

CONSTANT RESET_ACTIVE:std_logic:='1';

begin
-- purpose	:samples the MOSI data with a rising edge of the SCK
--	type		:memorizing
--	inputs 	:reset,clo
--	outputs	:
sync: process(reset,clk)
begin
	if reset=RESET_ACTIVE then
		shift_in<='0';
	elsif rising_edge(clk) then
		if sck_re='1' then
			shift_in<=sin;											-- sample with rising edge of sck
		end if;
	end if;
end process;
-- purpose	:implements the shift register
--	type		:memorizing
--	inputs 	:reset,clk
--	outputs	:
shift_register_seq: process(clk,reset)
begin
	if reset=RESET_ACTIVE then
		data_int<=(others=>'0');
	elsif rising_edge(clk) then
		data_int<=data_int_nxt;	
	end if;
end process;
-- purpose	:creates the next value for the shift register
--	type		:memoryless
--	inputs 	:Data_int,sr_reset,shift_enable,sck_fe,shift_in
--	outputs	:Data_int_nxt
shift_register_com: process(Data_int,sr_reset,shift_enable,sck_re,shift_in)
begin
	-- default statement
	Data_int_nxt<=Data_int;
	if sr_reset=RESET_ACTIVE then
		data_int_nxt<=(others=>'0');
	elsif shift_enable='1' and sck_re='1' then 		  -- shift with falling edge of sck
			data_int_nxt<=data_int(6 downto 0) & shift_in;
	end if;
end process;
-- purpose	:creates the data for parallel readout of the shift register
--	type		:memorizing
--	inputs 	:reset,clk
--	outputs	:
data_out_p: process (reset,clk)
begin
	if reset=RESET_ACTIVE then
		data_out<=(others=>'0');
	elsif rising_edge(clk) then
		data_out<=data_int(6 downto 0) & shift_in;
	end if;
end process;

end behavioral;