library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity slave_output_sr is
    Port ( 	Datain : in std_logic_vector(7 downto 0);
      	 	clk   : in std_logic;
		 		sck_fe: in std_logic;
		 		reset  : in std_logic;
           	load : in std_logic;
		 		enable: in std_logic;
           	miso_out : out std_logic);
end slave_output_sr;

architecture Behavioral of slave_output_sr is
CONSTANT RESET_ACTIVE:std_logic:='1';
signal slave_output_sr,slave_output_sr_nxt:std_logic_vector(7 downto 0);
signal miso:std_logic;
			  
begin
-- purpose	:implements the shift register
--	type		:memorizing
--	inputs 	:reset,clk
--	outputs	:
Shift_register_seq: process (reset,clk)
begin
	if (reset=RESET_ACTIVE) then
		slave_output_sr<=(others=>'0');
	elsif rising_edge(clk) then 		
		slave_output_sr<=slave_output_sr_nxt;	
	end if;
end process;
-- purpose	:creates the next value for the shift register
--	type		:memoryless
--	inputs 	:slave_output_sr,Load,Datain,sck_fe
--	outputs	:slave_output_sr_nxt
shift_register_com: process(slave_output_sr,Load,Datain,sck_fe,enable)
begin
	-- default statement
	slave_output_sr_nxt<=slave_output_sr;
  	if (Load='1') then
		slave_output_sr_nxt<=Datain;	
	elsif sck_fe='1' and enable='1' then
			slave_output_sr_nxt <= slave_output_sr(6 downto 0) & '0'; -- MSB first
	end if;
end process;

-- purpose	:multiplexer with desides which FF put its value on the MISO line
--	type		:memoryless
--	inputs 	:wordlength,slave_output_sr
--	outputs	:miso
miso<=slave_output_sr(7);
-- purpose	:activates the output if the slave is used,
--				 if the the master is active the line is tristate
--	type		:memoryless
--	inputs 	:enable,miso
--	outputs	:miso_out	
miso_out<=miso when enable='1' else 'Z';
													
end Behavioral;
