library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity slave_sm is
    Port ( clk : in std_logic;
           reset : in std_logic;
           shift_enable : out std_logic;
			  enable:in std_logic;
			  sck_fe:in std_logic;
			  sck_re:in std_logic;
			  ss_fe:in std_logic;
			  ss_re:in std_logic;
           reset_sr : out std_logic;
           busy : out std_logic;
			  loadxmit:out std_logic;
			  done:out std_logic);						  
end slave_sm;

architecture Behavioral of slave_sm is


type SPI_STATE_TYPE is (IDLE,REV_BIT,LAST_RISING_EDGE,FINISH_REV,READY);

signal spi_state,spi_state_nxt: SPI_STATE_TYPE;

signal bit_cnt  : std_logic_vector(2 downto 0);
signal bit_cnt_en: std_logic;
signal bit_cnt_load : std_logic;

signal bit_cnt_reload:std_logic_vector(2 downto 0);

CONSTANT RESET_ACTIVE: std_logic :='1';

begin

bit_cnt_reload<="111";	


-- counter for the xmittet Bits
counter : process(clk,bit_cnt_reload,bit_cnt_load,sck_re)
begin
	if bit_cnt_load='1' then
		bit_cnt<="111";	
	elsif rising_edge(clk) then
		if bit_cnt_en='1' and sck_re='1' then
			bit_cnt<=bit_cnt-1;
		end if;
	end if;
end process;

spi_sm_seq: process(clk,reset)
begin
	if(reset=RESET_ACTIVE) then
		spi_state<=IDLE;
	elsif rising_edge(clk) then
		spi_state<=spi_state_nxt;
	end if;
end process;

spi_sm_com: process(spi_state,bit_cnt,enable,ss_fe,ss_re,sck_re,sck_fe)
begin
-- default statements
spi_state_nxt<=spi_state;
shift_enable<='0';                      
loadxmit<='0';
bit_cnt_en<='0';
reset_sr<='0';
busy<='0';
bit_cnt_load<='0';
done<='0';
	case spi_state is
	  -- moore automat
		when IDLE =>	
			reset_sr<='1';
			bit_cnt_load<='1';
			loadxmit<='1';	
			if ss_fe='1' and enable='1' then					
				spi_state_nxt<=REV_BIT;
			end if;
		when REV_BIT =>
			shift_enable<='1';
			bit_cnt_en<='1';
			busy<='1';			
			if bit_cnt="000" then	
				spi_state_nxt<=LAST_RISING_EDGE;
			end if;
		when LAST_RISING_EDGE =>
			busy<='1';
			shift_enable<='1';
			if sck_re='1' then
				spi_state_nxt<=FINISH_REV;
			end if;
		when FINISH_REV=>
			busy<='1';
			if sck_fe='1' then	 		-- last sck cycle has finished
				spi_state_nxt<=READY;
			end if;
		when READY =>
			done<='1';
			if ss_re='1' then	 		  -- transmission from master finished
				spi_state_nxt<=IDLE;
			end if;	
		when others => spi_state_nxt<=IDLE;
	end case;
end process;


end Behavioral;
